Index: encoder/encoder.c
===================================================================
--- encoder/encoder.c	(revision 717)
+++ encoder/encoder.c	(working copy)
@@ -107,13 +107,23 @@
 {
     x264_param_t *param = &h->param;
     int i;
+	/* ME ADD : local i_first_mb, i_last_mb and i_last_frame_mb variables */
+	int i_first_mb, i_last_mb, i_last_frame_mb;
 
     /* First we fill all field */
     sh->sps = sps;
     sh->pps = pps;
 
-    sh->i_first_mb  = 0;
-    sh->i_last_mb   = h->sps->i_mb_width * h->sps->i_mb_height;
+	/* ME ADD : first_mb and last_mb computation */
+	i_first_mb = h->i_current_slice * h->param.i_slices_size;
+	i_last_mb = ( h->i_current_slice + 1 ) * h->param.i_slices_size;
+	i_last_frame_mb = h->sps->i_mb_width * h->sps->i_mb_height;
+	sh->i_first_mb  = i_first_mb;
+	sh->i_last_mb   = i_last_mb < i_last_frame_mb ? i_last_mb : i_last_frame_mb;
+	
     sh->i_pps_id    = pps->i_id;
 
     sh->i_frame_num = i_frame;
@@ -987,9 +997,8 @@
     if( i_nal_type == NAL_SLICE_IDR )
     {
         x264_slice_header_init( h, &h->sh, h->sps, h->pps, h->i_idr_pic_id, h->i_frame_num, i_global_qp );
-
-        /* increment id */
-        h->i_idr_pic_id = ( h->i_idr_pic_id + 1 ) % 65536;
+        /* increment id */ // Moved to the x264_slices_init function
+        //h->i_idr_pic_id = ( h->i_idr_pic_id + 1 ) % 65536;
     }
     else
     {
@@ -1020,6 +1029,34 @@
     x264_macroblock_slice_init( h );
 }
 
+/* ME ADD : Create slice header for each slice */
+static inline void x264_slices_init( x264_t *h, int i_nal_type, int i_global_qp)
+{
+	int i_s;
+	int i_frame_type;
+	i_frame_type = h->sh.i_type;
+	for( i_s = 0; i_s < h->param.i_slices_number; i_s++)
+	{
+		h->i_current_slice = i_s;
+		x264_slice_init( h, i_nal_type, i_global_qp );
+		memcpy( &h->shs[i_s], &h->sh, sizeof(x264_slice_header_t) );
+		memcpy( h->shs[i_s].sps, h->sh.sps, sizeof(h->sh.sps) );
+		memcpy( h->shs[i_s].pps, h->sh.pps, sizeof(h->sh.pps) );
+		memcpy( h->shs[i_s].i_delta_poc, h->sh.i_delta_poc, sizeof(h->sh.i_delta_poc) );
+		memcpy( h->shs[i_s].ref_pic_list_order, h->sh.ref_pic_list_order, sizeof(h->sh.ref_pic_list_order) );
+		h->shs[i_s].i_type = i_frame_type;
+	}
+	if( i_nal_type == NAL_SLICE_IDR )
+		h->i_idr_pic_id = ( h->i_idr_pic_id + 1 ) % 65536;
+}
+
 static void x264_slice_write( x264_t *h )
 {
     int i_skip;
@@ -1212,13 +1249,31 @@
 static int x264_slices_write( x264_t *h )
 {
     int i_frame_size;
+	/* ME ADD : i_s local variable */
+	int i_s;
 
 #if VISUALIZE
     if( h->param.b_visualize )
         x264_visualize_init( h );
 #endif
+	for( i_s = 0; i_s < h->param.i_slices_number; i_s++ )
+	{
+		h->i_current_slice = i_s;
+		memcpy( &h->sh, &h->shs[i_s], sizeof(x264_slice_header_t) );
+		memcpy( h->sh.sps, h->shs[i_s].sps, sizeof(h->shs[i_s].sps) );
+		memcpy( h->sh.pps, h->shs[i_s].pps, sizeof(h->shs[i_s].pps) );
+		memcpy( h->sh.i_delta_poc, h->shs[i_s].i_delta_poc, sizeof(h->shs[i_s].i_delta_poc) );
+		memcpy( h->sh.ref_pic_list_order, h->shs[i_s].ref_pic_list_order, sizeof(h->shs[i_s].ref_pic_list_order) );
+		x264_stack_align( x264_slice_write, h );
+	}
 
-    x264_stack_align( x264_slice_write, h );
     i_frame_size = h->out.nal[h->out.i_nal-1].i_payload;
     x264_fdec_filter_row( h, h->sps->i_mb_height );
 
@@ -1257,7 +1312,13 @@
     int     i_nal_ref_idc;
 
     int   i_global_qp;
+	
+	/* ME ADD : i_frame_type local variable */
+	int		i_frame_type;
+	/* ME ADD : i_current_slice local variable */
+	int		i_current_slice;
 
+	/* Multi-threading related code */
     if( h->param.i_threads > 1)
     {
         int i = ++h->i_thread_phase;
@@ -1371,31 +1432,36 @@
 
         i_nal_type    = NAL_SLICE_IDR;
         i_nal_ref_idc = NAL_PRIORITY_HIGHEST;
-        h->sh.i_type = SLICE_TYPE_I;
+		i_frame_type = SLICE_TYPE_I;
     }
     else if( h->fenc->i_type == X264_TYPE_I )
     {
         i_nal_type    = NAL_SLICE;
         i_nal_ref_idc = NAL_PRIORITY_HIGH; /* Not completely true but for now it is (as all I/P are kept as ref)*/
-        h->sh.i_type = SLICE_TYPE_I;
+		i_frame_type = SLICE_TYPE_I;
     }
     else if( h->fenc->i_type == X264_TYPE_P )
     {
         i_nal_type    = NAL_SLICE;
         i_nal_ref_idc = NAL_PRIORITY_HIGH; /* Not completely true but for now it is (as all I/P are kept as ref)*/
-        h->sh.i_type = SLICE_TYPE_P;
+		i_frame_type = SLICE_TYPE_P;
     }
     else if( h->fenc->i_type == X264_TYPE_BREF )
     {
         i_nal_type    = NAL_SLICE;
         i_nal_ref_idc = NAL_PRIORITY_HIGH; /* maybe add MMCO to forget it? -> low */
-        h->sh.i_type = SLICE_TYPE_B;
+		i_frame_type = SLICE_TYPE_B;
     }
     else    /* B frame */
     {
         i_nal_type    = NAL_SLICE;
         i_nal_ref_idc = NAL_PRIORITY_DISPOSABLE;
-        h->sh.i_type = SLICE_TYPE_B;
+		i_frame_type = SLICE_TYPE_B;
     }
 
     h->fdec->i_poc =
@@ -1406,6 +1472,10 @@
     h->fdec->b_kept_as_ref = i_nal_ref_idc != NAL_PRIORITY_DISPOSABLE && h->param.i_keyint_max > 1;
 
 
+	/* ------------------- Setup slice context ----------------------------- */
+	/* ME ADD : Initialize h->sh variable to point to first element of h->shs array */
+	i_current_slice = 0;
+	h->sh.i_type = i_frame_type;
 
     /* ------------------- Init                ----------------------------- */
     /* build ref list 0/1 */
@@ -1421,11 +1491,11 @@
     if( h->sh.i_type == SLICE_TYPE_B )
         x264_macroblock_bipred_init( h );
 
-    /* ------------------------ Create slice header  ----------------------- */
-    x264_slice_init( h, i_nal_type, i_global_qp );
+    /* ------------------------ Create slices header  ----------------------- */
+	x264_slices_init( h, i_nal_type, i_global_qp );
 
-    if( i_nal_ref_idc != NAL_PRIORITY_DISPOSABLE )
-        h->i_frame_num++;
+	if( i_nal_ref_idc != NAL_PRIORITY_DISPOSABLE )
+		h->i_frame_num++;
 
     /* ---------------------- Write the bitstream -------------------------- */
     /* Init bitstream context */
@@ -1435,11 +1505,11 @@
     if(h->param.b_aud){
         int pic_type;
 
-        if(h->sh.i_type == SLICE_TYPE_I)
+        if(i_frame_type == SLICE_TYPE_I)
             pic_type = 0;
-        else if(h->sh.i_type == SLICE_TYPE_P)
+        else if(i_frame_type == SLICE_TYPE_P)
             pic_type = 1;
-        else if(h->sh.i_type == SLICE_TYPE_B)
+        else if(i_frame_type == SLICE_TYPE_B)
             pic_type = 2;
         else
             pic_type = 7;
@@ -1717,11 +1787,12 @@
     psz_message[79] = '\0';
     
     x264_log( h, X264_LOG_DEBUG,
-                  "frame=%4d QP=%i NAL=%d Slice:%c Poc:%-3d I:%-4d P:%-4d SKIP:%-4d size=%d bytes%s\n",
+                  "frame=%4d QP=%i NAL=%d Slice:%c Slice#:%d Poc:%-3d I:%-4d P:%-4d SKIP:%-4d size=%d bytes%s\n",
               h->i_frame,
               h->fdec->i_qpplus1 - 1,
               h->i_nal_ref_idc,
               h->sh.i_type == SLICE_TYPE_I ? 'I' : (h->sh.i_type == SLICE_TYPE_P ? 'P' : 'B' ),
+			  h->i_current_slice,
               h->fdec->i_poc,
               h->stat.frame.i_mb_count_i,
               h->stat.frame.i_mb_count_p,
Index: x264.c
===================================================================
--- x264.c	(revision 717)
+++ x264.c	(working copy)
@@ -296,6 +296,7 @@
     H0( "      --fps <float|rational>  Specify framerate\n" );
     H0( "      --seek <integer>        First frame to encode\n" );
     H0( "      --frames <integer>      Maximum number of frames to encode\n" );
+    H0( "      --slice-size <integer>  Maximum size of slice in macroblocks\n" );
     H0( "      --level <string>        Specify level (as defined by Annex A)\n" );
     H0( "\n" );
     H0( "  -v, --verbose               Print stats for each frame\n" );
@@ -355,6 +356,7 @@
 #define OPT_PROGRESS 261
 #define OPT_VISUALIZE 262
 #define OPT_LONGHELP 263
+#define OPT_SLICE_SIZE 264
 
         static struct option long_options[] =
         {
@@ -386,6 +388,7 @@
             { "sar",     required_argument, NULL, 0 },
             { "fps",     required_argument, NULL, 0 },
             { "frames",  required_argument, NULL, OPT_FRAMES },
+            { "slice-size",  required_argument, NULL, OPT_SLICE_SIZE },
             { "seek",    required_argument, NULL, OPT_SEEK },
             { "output",  required_argument, NULL, 'o' },
             { "analyse", required_argument, NULL, 0 },
@@ -484,6 +487,9 @@
             case OPT_FRAMES:
                 param->i_frame_total = atoi( optarg );
                 break;
+            case OPT_SLICE_SIZE:
+                param->i_slices_size = atoi( optarg );
+                break;
             case OPT_SEEK:
                 opt->i_seek = atoi( optarg );
                 break;
@@ -668,6 +674,18 @@
         }
     }
 #endif
+	
+	/* ME ADD : Define default values for slice numbers/size */
+	// FIX ME : Add the slice size parameter to the command line parameters
+	int i_mb_qty;
+	i_mb_qty = param->i_width / 16 * param->i_height / 16;
+	if( param->i_slices_size == 0 )
+		param->i_slices_size = i_mb_qty;
+	else
+		param->i_slices_number = (int) ( ceil( (double) ( i_mb_qty ) / (double) ( param->i_slices_size ) ) );
+	fprintf( stderr, "x264 [info]: Slices number:%d\n", param->i_slices_number );
+	fprintf( stderr, "x264 [info]: Slices size:%d\n", param->i_slices_size );
 
     return 0;
 }
Index: common/common.c
===================================================================
--- common/common.c	(revision 717)
+++ common/common.c	(working copy)
@@ -138,6 +138,10 @@
 
     param->b_repeat_headers = 1;
     param->b_aud = 0;
+	
+	/* Slice parameters */
+	param->i_slices_number = 1;
+	param->i_slices_size = 0;
 }
 
 static int parse_enum( const char *arg, const char * const *names, int *dst )
Index: common/common.h
===================================================================
--- common/common.h	(revision 717)
+++ common/common.h	(working copy)
@@ -288,6 +288,10 @@
     uint32_t        nr_count[2];
 
     /* Slice header */
+	/* ME ADD : New Slice-related code */
+	/* Maximum slice number allowed is 8 */
+	x264_slice_header_t shs[8];
+	int i_current_slice;
     x264_slice_header_t sh;
 
     /* cabac context */
Index: x264.h
===================================================================
--- x264.h	(revision 717)
+++ x264.h	(working copy)
@@ -278,6 +278,10 @@
     int b_aud;                  /* generate access unit delimiters */
     int b_repeat_headers;       /* put SPS/PPS before each keyframe */
     int i_sps_id;               /* SPS and PPS id number */
+	
+	/* Slice parameters */
+	int i_slices_number;		/* number of slices per frame */
+	int i_slices_size;			/* size of each slice in macroblock */
 } x264_param_t;
 
 typedef struct {
